const channelsID = require('../channels_conf.json');
const complotDB = require('../database/ComplotDB.js');

const CATEGORY = {
    ACTION : 0,
    REASON : 1,
    ACTOR : 2,
    SOURCE : 3,
    SENTENCE : 4
};

class VoteParser{
    
    constructor(client){
        this.client = client;
        this.upVote = '👍';
        this.score = 3;
        this.categories = new Map([
            [channelsID.actor, CATEGORY.ACTOR],
            [channelsID.reason, CATEGORY.REASON],
            [channelsID.action, CATEGORY.ACTION],
            [channelsID.source, CATEGORY.SOURCE],
            [channelsID.sentence, CATEGORY.SENTENCE]
        ]);

        this.#resetStoredMessages();

        this.channelBase = this.client.channels.cache.get(channelsID.base);
    }

    #resetStoredMessages() {
        this.infoMessagesActor = [];
        this.infoMessagesReason = [];
        this.infoMessagesAction = [];
        this.infoMessagesSource = [];
        this.infoMessagesSentence = [];
    }

    setScore(score) {
        this.score = score;
    }

    parse(message){
        if(message.author.id === this.client.application.id){
            return;
        }
        if(this.categories.has(message.channelId)){
            const category = this.categories.get(message.channelId);
            const reactions = message.reactions.resolve(this.upVote);
            if(reactions && reactions.count >= this.score){
                this.add(message, category);
            }
        }
    }

    categoryCase(category, content){
        if(category === CATEGORY.SOURCE || category === CATEGORY.SENTENCE){
            return content;
        }

        const char = content.charAt(0);
        const reste = content.slice(1);
        content = (category === CATEGORY.ACTOR) ? char.toUpperCase() + reste: char.toLowerCase() + reste;
        return content;
    }

    async add(message, category){
        let content = this.categoryCase(category, message.content);
        
        this.addFormatedString(content, category);
        message.delete();
        message.channel.send(`"${content}" ajouté ! \n Par <@${message.author.id}>`);
        
        this.updateBase(category);
    }

    #splitMessage(text) {
        let textLines = text.split("\n");
        let splitArray = [];
        let currentTextPart = "";

        for (let line of textLines) {
            if ((currentTextPart + line).length > 1950) {
                splitArray.push(currentTextPart);
                currentTextPart = line;
            } else currentTextPart += ("\n" + line);
        }

        splitArray.push(currentTextPart);
        return splitArray;
    }

    async updateBase(category){
        // Gosh this fucking code needs to be factored, so many repetitions

        const actor = this.#splitMessage(complotDB.actorToString()).map(el => '```' + el + '```');
        const action = this.#splitMessage(complotDB.actionToString()).map(el => '```' + el + '```');
        const reason = this.#splitMessage(complotDB.reasonToString()).map(el => '```' + el + '```');
        const source = this.#splitMessage(complotDB.sourceToString()).map(el => '```' + el + '```');
        const sentence = this.#splitMessage(complotDB.sentenceToString()).map(el => '```' + el + '```');

        if (!(this.infoMessagesActor instanceof Array) || this.infoMessagesActor.length === 0) {
            this.#resetStoredMessages();
            await this.deleteAllMessageInfo();

            for (let messagePart of actor)
                this.infoMessagesActor.push(await this.channelBase.send(messagePart));
            for (let messagePart of action)
                this.infoMessagesAction.push(await this.channelBase.send(messagePart));
            for (let messagePart of reason)
                this.infoMessagesReason.push(await this.channelBase.send(messagePart));
            for (let messagePart of source)
                this.infoMessagesSource.push(await this.channelBase.send(messagePart));
            for (let messagePart of sentence)
                this.infoMessagesSentence.push(await this.channelBase.send(messagePart));
        } else {
            switch (category) {
                case CATEGORY.ACTOR:
                    // If we had more messages before the update, we get rid of them here
                    this.infoMessagesActor.splice(actor.length)
                    // If in the opposite we have more messages now, we re-send everything
                    if (actor.length > this.infoMessagesAction.length) {
                        this.#resetStoredMessages();
                        return await this.updateBase(category);
                    }
                    // Now we edit!
                    for (let i = 0; i < actor.length; i++)
                        this.infoMessagesActor[i].edit(actor[i]);

                    break;
                case CATEGORY.ACTION:
                    this.infoMessagesAction.splice(action.length)
                    if (action.length > this.infoMessagesAction.length) {
                        this.#resetStoredMessages();
                        return await this.updateBase(category);
                    }
                    for (let i = 0; i < action.length; i++)
                        this.infoMessagesAction[i].edit(action[i]);
                    break;
                case CATEGORY.REASON:
                    this.infoMessagesReason.splice(reason.length)
                    if (reason.length > this.infoMessagesReason.length) {
                        this.#resetStoredMessages();
                        return await this.updateBase(category);
                    }
                    for (let i = 0; i < reason.length; i++)
                        this.infoMessagesReason[i].edit(reason[i]);
                    break;
                case CATEGORY.SOURCE:
                    this.infoMessagesSource.splice(source.length)
                    if (source.length > this.infoMessagesSource.length) {
                        this.#resetStoredMessages();
                        return await this.updateBase(category);
                    }
                    for (let i = 0; i < source.length; i++)
                        this.infoMessagesSource[i].edit(source[i]);
                    break;
                case CATEGORY.SENTENCE:
                    this.infoMessagesSentence.splice(sentence.length)
                    if (sentence.length > this.infoMessagesSentence.length) {
                        this.#resetStoredMessages();
                        return await this.updateBase(category);
                    }
                    for (let i = 0; i < sentence.length; i++)
                        this.infoMessagesSentence[i].edit(sentence[i]);
                    break;
            }
        }
    }


    addFormatedString(messageString, category){
        switch (category){
            case CATEGORY.ACTOR: complotDB.addActor(messageString); break;
            case CATEGORY.ACTION: complotDB.addAction(messageString); break;
            case CATEGORY.REASON: complotDB.addReason(messageString); break;
            case CATEGORY.SOURCE: complotDB.addSource(messageString); break;
            case CATEGORY.SENTENCE: complotDB.addSentence(messageString); break;
        }
    }

    async deleteAllMessageInfo(){

        let messages = await this.channelBase.messages.fetch();
        let workers = [];
        messages.forEach(m=>{
            workers.push(m.delete());
        });
        await Promise.all(workers);
    }

}

module.exports = VoteParser;